/* cbar.c */

#include "vpt.h"
#include "att_table.h"

#define MAXCBAR 20

static char changebar,		/* change bar status (0==off)	   */
			cfetched,
			cbardata[MAXCBAR],	/* subsitution data for change bar */
			cbarmargin;			/* margin for bars (lrio)		   */

static Att_table cb_att_table;

/*	============================================================
	Set the change bar status \CB+ \CB-
	
	This is called from input, it only changes the flag.
	
	Note: When the first change bar enable is received the font is
		  set to the current body font. A different font can be selected
		  by embedding the font in the change bar text.
*/

void
setcbarstatus(void)
{
	KCHAR c=getkey();
	static FLAG first=0;
	
	if(c=='+') changebar=cfetched=1;
	else if(c=='-') changebar=0;
	else error("Illegal change bar flag (use '+' or '-'");
	
	if(!first)
	{
		cb_att_table.cfont=getbodyfont();
		first++;
	}
}

/*	============================================================
	Return the status of the change bar flag. The 'cfetched' var is
	needed since the same line can turn a cbar on and off. If the
	bar status has not been checked since it was set (as possibly
	cleared again) it's the same as if changebar is still on.
	
	Pass the current page status (odd/even)
	
	Returns: 0 - no bar set OR in column mode
			 1 - need bar in left margin
			 2 - need bar in right margin
			 3 - need bar in both margins
*/

int
getcbarstatus(int oddeven)
{
	int status=0;
	
	if(!numcols && (changebar || cfetched)) switch(f_tolower(cbarmargin))
	{
		case 'l': 
			status=1;	
			break;	   /* left margin */
		
		case 'r': 
			status=2;	
			break;	   /* right margin */
		
		case 'b': 
			status=3;	
			break;	   /* both margins */
		
		case 'i':						 /* inner margin */
			status=(oddeven) ? 1 : 2;
			break;
		
		case 'o':						 /* outer margin */
			status=(oddeven) ? 2 : 1;
			break;
			
		default:
			break;		/* should never happen */
	}
	cfetched=0;
	return status;
}

/*	============================================================
	Set the character string to use to print change bars. Called
	from command parser CB[lrio]={bar data}
	
	cbl=[|]  sets bars to left margin, use "|" as data
	cbl		 set bars to left margin, use existing data
	
	NOTE: the equal sign is necessary to change data!
*/

void
setcbar(void)
{
	KCHAR c;
	
	cbarmargin='l';				 /* assume default margin  */
	c=getkey();				 /* check for option flags */
	if(index("lroib", f_tolower(c))) cbarmargin=f_tolower(c);
	else returnkey(c);
	
	c=getnonb();
	if(c=='=')
		getdstring(cbardata, MAXCBAR-1, "change bar definition");
	else returnkey(c);
}

/*	============================================================
	Print a change bar. The margin size passed is used as a "left
	margin" for the bar as well as to set the max size the bar
	can use. For left margins the caller will have to pad out to
	the text start pos. The total size printed (margin+bar) is
	returned.
*/

int
docbar(int msize)
{
	KCHAR buff[200];
	int mdone, m;
	char hs;
	
	hs=hardspace;  /* disable the hardspace char */
	hardspace=0;
	
	mdone=domargin(msize);  /* do the leading margin */
	
	pushkbuff();
	m=addmacro(cbardata);
	
	mdone+=fillbuff(buff, buff+sizeof(buff)-2, msize, 0, 0,
						&cb_att_table, FMT_CBAR);	
	printline(buff, &cb_att_table);
	
	hardspace=hs;  /* reset the hardspace char */
	
	while(getkey()!=INT_DELIM);
	killmacro(m);
	pullkbuff();
	
	return mdone;
}
