/* cmd.c,  command line processor */

#include "vpt.h"
#include <ctype.h>

/*	============================================================
	Process a command line. The leading command line character (.)
	has been skipped, but multiples are permitted..
*/

void
docmdline(void)
{
	KCHAR cmdkey1, cmdkey2;
	u_int t;
	
	doingcmd=1;
	
	for( ; ; )
	{
		/* first try for some single key commands */
		
		cmdkey1=getnonb();
		
		if(cmdkey1==INT_CR) cmdkey1=CR;
		
		if(!cmdkey1 || cmdkey1==CR) break; /* end of command line */
		if(cmdkey1==dot) continue;	  /* unnecessary extra dot, ignore */
		
		/*	check for embedded tokens.... attribute/font on/off codes are
			sent to routine in body which will set the flags in the main
			body table; all other tokens (and there shouldn't be any!) are
			ignored.
		*/
		
		if(istoken(cmdkey1))
		{
			if(cmdkey1==INT_WORDATT) getkey();	 /* ignore word attrs */
			
			else	if(is_attron(cmdkey1)	||
					is_attroff(cmdkey1)		||
					is_font(cmdkey1)) setbodyatt(cmdkey1);
			
			else if(cmdkey1==INT_NOTE) extractnote();
			
			else if(cmdkey1==INT_PAD_ON || cmdkey1==INT_PAD_OFF)
				setcharpadding(cmdkey1);
			
			continue;
		}
		
		/* First we check 2 character options. */
		
		cmdkey2=getkey();
		
		/*	handle "set attributes" differently--we have to check
			the 2nd char to see if it is + or - to see if this is
			a define attribute command
		*/
		
		if(cmdkey2=='-' || cmdkey2=='+')
		{
			setattr(cmdkey1, cmdkey2);
			continue;
		}

		switch((tolower(cmdkey1)<<8) + tolower(cmdkey2))
		{
			dcase('c', 'b'):		/* set change bar, cbar.c */
				setcbar();
				continue;

			dcase('c', 'c'):		/* set escape char (\) */
				skipequal();
				control=getAval();
				continue;

			dcase('c', 'd'):
				changedir();		/* change directory, output.c */
				continue;
			
			dcase('c', 'h'):		/* set halfline codes, crs.c */
				sethfcodes();
				continue;

			dcase('c', 'l'):		 /* set ctrl line char (.) */
				skipequal();
				dot=getBval(' ');
				continue;
			
			dcase('c', 'p'):		/* set partline codes, crs.c */
				setplcodes();
				continue;

			dcase('c', 's'):		/* inter character spacing, format.c*/
				setispacing();
				continue;

			dcase('c', 'o'):		/* set # columns, columns.c */
				setncols();
				continue;
	
			dcase('c', 'r'):		/* set CR codes, crs.c */
				setcrcodes();
				continue;
				
			dcase('d', 'p'):		/* dumb printer flag, output.c */
				set_dumbprint();
				continue;

			dcase('e', 'c'):		/* set error continue flag */
				seterrflag(getonoff());
				continue;

			dcase('e', 'h'):		/* echo text line, strings.c */
				echotext();
				continue;
			
			dcase('e', 'r'):		/* do error, error.c */
				userr();
				continue;

			dcase('f', '#'):		/* set number format, numbers.c */
				setnumf();
				continue;
				
			dcase('g', 'h'):		/* set ghost hyphen char */
				skipequal();
				ghosthy=getAval();
				continue;
						
			dcase('g', 't'):		/* goto command */
				dogoto();
				continue;
			
			dcase('g', 'u'):		/* set gutter width, margins.c */
				setgutter();
				continue;

			dcase('h', 'd'):		/* define a header, headers.c */
				sethead();
				continue;
			
			dcase('h', 'p'):		/* enable/disable header printing */
				header_printing();
				continue;
			
			dcase('h', 's'):		/* hardspace char */
				skipequal();
				hardspace=getAval();
				continue;

			dcase('i', 'b'):		/* set dual indent, indents.c */
				setbothid();
				continue;
			
			dcase('i', 'c'):		/* set center indent, indents.c */
				setcenterid();
				continue;
			
			dcase('i', 'd'):		/* set delayed indent, indents.c */
				setdelayid();
				continue;

			dcase('i', 'f'):		/* do an IF, cmd.c */
				ifthen();
				continue;
			
			dcase('i', 'l'):		/* set left indent, indents.c */
				setleftid();
				continue;
			
			dcase('i', 'r'):		/* set right indent, indents.c */
				setrightid();
				continue;
			
			dcase('i', 't'):		/* set temp indent, indents.c */
				settempid();
				continue;
			
			dcase('j', 'c'):		/* justify CR lines, format.c */
				set_jcline();
				continue;
			
			dcase('j', 'm'):		/* justify mode, format.c */
				set_justmode();
				continue;
			
			dcase('l', 'a'):		/* line adjust flag, format.c */
				set_linead();
				continue;
			
			dcase('l', 'c'):		/* set current line */
				skipequal();
				currline=getBval(0);
				continue;

			dcase('l', 'f'):		/* set line format, format.c */
				setlineformat();
				continue;
			
			dcase('l', 'p'):		/* set lines/page */
			
				if(colactive()) columnerror("Lines/page");
				skipequal();
				t=line_page;			/* orig lines/page */
				line_page=getBval(0);	/* set lines/page */
				if(getbotmarg()>=t) changebotmarg(line_page-1);
				continue;

			dcase('l', 's'):		/* set line spacing, crs.c */
				setlinespace();
				continue;

			dcase('m', 'b'):		/* set bottom margin, margins.c */
				setbotmarg();
				continue;

			dcase('m', 'n'):		/* set marginal note margin, mnotes.c */
				setmargnote();
				continue;
			
			dcase('m', 't'):		/* set top margin, margins.c */
				settopmarg();
				continue;
			
			dcase('m', 'l'):		/* set left margin, margins.c */
				setlmarg();
				continue;
			
			dcase('m', 'r'):		/* set right margin, margins.c */
				setrmarg();
				continue;

			dcase('m', 's'):		/* set microspacing constant */
				skipequal();
				t=getval(0);
				
				if(t!=msperin)
				{
					ms_font(t);	// abort if any fonts defined
					ms_state(t);	// abort if states are active
					
					ms_tabcols(t);	// adjust tabcolumn and ichar size
					ms_margins(t);	// adjust margin sizes
					ms_mnotes(t);	// adjust marginal note size/gutter
					ms_defkeys(t);	// adjust def keys sizes
					ms_fnote(t);	// adjust footnote indent
					ms_indents(t);	// adjust the indents
					ms_smallspace(t); // adjust small space size
					ms_tabs(t);	// adjust tab stops
					ms_prefix(t);	// adjust prefix/suffix size
					
					msperin=t;
				}
				continue;
			
			dcase('n', '$'):		/* define number string, numbers.c */
				setnumstr();
				continue;

			dcase('n', '1'):		/* set footnote string 1, fnotes.c */
				setnum1();
				continue;
			
			dcase('n', '2'):		/* set footnote string 2, fnotes.c */
				setnum2();
				continue;
			
			dcase('n', '3'):		/* set footnote string 3, fnotes.c */
				setnum3();
				continue;
			
			dcase('n', '4'):		/* set footnote string 4, fnotes.c */
				setnum4();
				continue;
			
			dcase('n', '5'):		/* set footnote string 5, fnotes.c */
				setnum5();
				continue;

			dcase('n', 'f'):		/* set end/footnote, fnotes.c */
				setnotefm();
				continue;
			
			dcase('n', 'l'):		/* need lines */
				skipequal();
				pgifneed(getBval(0));
				continue;
			
			dcase('n', 'p'):		/* print endnotes, fnotes.c */
				printnotes();
				continue;
			
			dcase('n', 's'):		/* set footnote spacing, fnotes.c */
				setnotesp();
				continue;

			dcase('o', 'a'):		/* open file for \a[text], index.c */
				openauxf();
				continue;
			
			dcase('o', 'c'):		/* open file for \c[text], index.c */
				opentocf();
				continue;

			dcase('o', 's'):		/* do OS call, output.c */
				doshell();
				continue;
			
			dcase('o', 'x'):		/* open file of \x[text], index.c */
				openinxf();
				continue;

			dcase('p', 'b'):		/* set page bottom codes, page.c */
				setpbcodes();
				continue;

			dcase('p', 'c'):		/* start new column, columns.c */
				startcol();
				continue;

			dcase('p', 'e'):		/* set line prefix codes, prefix.c */
				setls();
				continue;
			
			dcase('p', 'i'):		/* do printer init codes, codes.c */
				dopicodes();
				continue;
			
			dcase('p', 'g'):		/* start new page, page.c */
				startpage();
				continue;
			
			dcase('p', 'm'):		/* set pad mode char size */
				setpadcharsize();
				continue;
			
			dcase('p', 'p'):		/* set page pause flag, wait.c */
				setpauseflag();
				continue;
			
			dcase('p', 'r'):		/* include [text] in text, miscmds.c */
				printinc();
				continue;
			
			dcase('p', 's'):		/* set line prefix, prefix.c */
				setlp();
				continue;

			dcase('p', 't'):		/* turn print on/off */
				printoff=!getonoff();
				setprintflag();
				continue;

			dcase('q', 's'):		/* set quit codes, error.c */
				setqscodes();
				continue;

			dcase('q', 'u'):		/* quit vprint, error.c */
				quit();
				continue;
			
			dcase('r', 'e'):		/* repeat end, input.c */
				repeatend();
				continue;
			
			dcase('r', 'p'):		/* repeat start, input.c */
				repeatstart();
				continue;

			dcase('s', '?'):		/* open aux input, input.c */
				openauxinp();
				continue;
			
				
			dcase('s', 'f'):	/* set the soft font dir name, laser.c */
				set_soft_font_dir();
				continue;

			dcase('s', 'i'):		/* switch input file, input.c */
				switchin();
				continue;
			
			dcase('s', 'l'):		/* sleep/delay, output.c */
				zzzz();
				continue;
			
			dcase('s', 'o'):		/* open output file, output.c */
				switchout();
				continue;
			
			dcase('s', 'p'):		/* do blank lines, miscmds.c */
				do_n_lines();
				continue;

			dcase('s', 'r'):		/* restore state, state.c */
				staterest();
				continue;
			
			dcase('s', 's'):	/* set small space codes, smallspace.c */
				setss();
				continue;

			dcase('s', 'v'):		/* save state, state.c */
				statesave();
				continue;
			
			dcase('t', 'r'):		/* translate table on/off */
				translate=getonoff();
				continue;

			dcase('t', 'b'):	/* set column width for tabs, format.c */
				settabcolumn();
				continue;

			dcase('t', 'f'):		/* set tab fill char, miscmds.c */
				settabfill();
				continue;
			
			dcase('t', 't'):		/* define translation, translate.c */
				def_trtbl();
				continue;
			
			dcase('w', '0'):		/* define wait messages, wait.c */
			dcase('w', '1'):
			dcase('w', '2'):
			dcase('w', '3'):
			dcase('w', '4'):
			dcase('w', '5'):
			dcase('w', '6'):
			dcase('w', '7'):
			dcase('w', '8'):
			dcase('w', '9'):
				setwaitmsg(cmdkey2-'0');
				continue;

			dcase('w', 'c'):		/* define wait codes, wait.c */
				setwcodes();
				continue;
			
			dcase('w', 'd'):		/* set widow lines, body.c */
				set_widowlines();
				continue;

			dcase('w', 'h'):		/* define a when, when.c */
				def_when();
				continue;
			
			dcase('x', 'd'):		/* index file delim, index.c */
				setindexdlm();
				continue;
				
			default:
				break;		/* drop to single key switch below */
		}
		
		returnkey(cmdkey2);
		
		/* do single key commands. Most of these need a 2nd arg... */
		
		switch(f_tolower(cmdkey1))
		{
			case ';':			/* skip delim */
				continue;
			
		  	case '*':			/* comment line */
				incomment++;
				while((cmdkey1=getkey()) && cmdkey1!=CR);
				incomment--;
				returnkey(cmdkey1);
				continue;
			
			case ':':			/* skip over label */
				incomment++;
				while((cmdkey1=getkey()))
				{
					if(istoken(cmdkey1)) error("Illegal label");
					if(cmdkey1==CR || cmdkey1==SPACE) break;
				}
				incomment--;
				returnkey(cmdkey1);
				continue;
			
			case '%':			/* set defined key seq, defkeys.c */
				setdefkeys();
				continue;
			
			case 'f':			/* define font, font.c */
				setfont();
				continue;
			
			case '@':			/* define a macro, macros.c */
			case '$':
				defmacro();
				continue;
			
			case '#':			/* set number reg, numbers.c */
				setnum();
				continue;
			
			case 't':			/* set tab stop, tabs.c */
				settab();
				continue;
			
			default:
				error("%c%c: unknown command", cmdkey1, cmdkey2);
		  }
	 }
	 doingcmd=0;
}
