/* defkeys.c */

#include "vpt.h"
#include "codes.h"
#include "att_table.h"

/*	defkeys[] is an array of pointers to the structures used to store
	defined key presses. Note that the number of codes for a defined
	key seq is 254, not the normal 24 for fonts, etc.
*/

static struct
{
	u_int	dsize;
	u_char	dflag;		/* if 0, use size, else use font offset		*/
	Codes	*dcodes;
}defkeys[NUM_DKEYS];

/*	============================================================
	Tokenize an embedded defined key
*/

void
tokdkey(void)
{
	int t;
	
	if((t=getregnum(getkey()))==-1) error("Unknown defined key name");
	returnkey(INT_DKEY+t);
}

/*	============================================================
	Set a defined key sequence.
	
	The defined keys are %0..%9, %a..%z
	
	.%x[=] width, codes
	
	if width==!value	then this is taken as a table offset rather
						than an absolute value.
	
	Upto 254 codes can be used. Memory is allocated dynamically for the
	defintions.
*/

void
setdefkeys(void)
{
	int keynum;
	u_char bangflag;
	KCHAR c;
	u_int keysize;
	
	if((keynum=getregnum(getkey()))==-1) error("Unknown defined key buffer");
	
	/* Get size.
		If !x then bangflag=char, keysize=??; if nn keysize=x, bangflag=0
	*/
	
	skipequal();
	if((c=getnonb())=='!')
	{
		bangflag=getval(0);
		if(bangflag<' ' && bangflag !=INT_HSPACE && bangflag!=INT_GH)
			error("Illegal defined key size/char offset");
	}
	else
	{
		returnkey(c);
		bangflag=0;
		keysize=getval(0);
	}
	
	setcodes(&defkeys[keynum].dcodes,"defined key");
	defkeys[keynum].dsize = keysize;
	defkeys[keynum].dflag = bangflag;
}

/*	============================================================
	Send a defined key sequence.
	Called from printline when a defkey token is encountered.
*/

void
sendDkey(int n)
{
	docodes(defkeys[n].dcodes);
}

/*	============================================================
	Determine the size of a defined key token. Called from getcsize()
*/

int
getdksize(int n, Att_table *att)
{
	int retvalue;
	
	if(defkeys[n].dcodes==NULL) retvalue=0;	/* key not defined */
	
	else if(defkeys[n].dflag)
		retvalue = getcsize((KCHAR)defkeys[n].dflag, att);
	
	else retvalue = defkeys[n].dsize;
	
	return retvalue;
}

/*	======================================================================
	Adjust the size of the defined keys after a MS=
*/

void
ms_defkeys(int n)
{
	int t;
	
	for(t=0; t<arraysize(defkeys); t++)
	{
		defkeys[t].dsize=ms_adjust(defkeys[t].dsize, n);
	}
}
