/* header.c */

#include "vpt.h"
#include "att_table.h"

/* struture to hold definined header info */

typedef struct hptr
{
	struct hptr *next;		  	/* pointer to next entry				 */
	char *eventxt, *oddtxt; 	/* pointers to text for odd & even pages */
	char linenum;		  		/* line number for this header			 */
	u_char htabfill;			/* tab fill char for this header		 */
}Hptr;

static Hptr *root, *last;

/*	Attributes/font for the header. This is passed for header to header,
	so header attributes continue from line to line...
*/

static Att_table head_attrs;

static FLAG headers_off;	/* set if headers are disabled */

/* ============================================================
   Define a header
   
   .HD[o,e]nn=[*tabfill] <text>
*/

void
sethead(void)
{
	char headtext[200], *p;
	KCHAR c, eo;
	u_int num;
	Hptr *temp;
	u_char tf;
	static char *emsg="No memory for header definition";
	
	/* get optional E or O  */
	
	c=getkey();
	eo=f_toupper(c);
	if(eo!='E' && eo!='O')
	{
		returnkey(c);
		eo=0;
	}
	
	/* make sure line number follows */
	
	returnkey(c=getnonb());
	if(!f_isdigit(c)) error("Header line number needed");
	
	/*	Shove the current font token at the start of the header. This
		will set the header to the currently selected font--if you want
		to use something else then just start the header def with it.
	*/
	
	untokenize(headtext, getbodyfont()+INT_FONT);
	
	/* get the line number */
	
	num=getval(0);
	skipequal();
	
	/* see if we need a special tab fill character for this line */
	
	if((c=getkey())=='*')
	{
		tf=getval(0);
	}
	else
	{
		returnkey(c);
		tf=' ';
	}
	
	getdstring(strend(headtext), arraysize(headtext)-7, "header definition");
	
	/*	Delete prior definitions (not structure pointer). If this is an odd
		or both line we have to delete the prior odd definition, if this is
		an even line then we have to the delete prior even definitions
	*/
	
	for(temp=root; temp; temp=temp->next)
	{
		if(temp->linenum==num)
		{
			if((eo=='O' || !eo) && temp->oddtxt)  /* doing odd/both */
			{
				free(temp->oddtxt);					/* delete old odd */
				temp->oddtxt=0;
			}
			if((eo=='E' || !eo) && temp->eventxt)  /* doing even/both */
			{
				free(temp->eventxt);				 /* delete old even */
				temp->eventxt=0;
			}
			break;
		}
	}
	
	/*	this is a new header line definition. Get a structure pointer
		and set up the links.
	*/
	
	if(!temp)
	{
		if((temp=(Hptr *)malloc(sizeof(Hptr)))==NULL)
			terminate(emsg);
		
		temp->linenum=num;
		temp->oddtxt=temp->eventxt=0;
		
		if(last==0)
		{
			temp->next=0;
			last=root=temp;
		}
		else
		{
			last->next=temp;
			temp->next=0;
			last=temp;
		}
	}
	
	/* get memory for the text and save it */
	
	p=strdup(headtext);
	
	if(eo=='E' || !eo) temp->eventxt=p;
	else temp->oddtxt=p;
	
	temp->htabfill=tf;	 /* save new tab fill character */
}


/* ============================================================
   Do a header line.
   
   The current line number is passed.
   If we find the header then we shove it to input via a macro.
   The string is terminated with a CR, which means that after the line
   is formatted and printed control will return here. We then read the
   input until we get our special delimiter--cleaning up the input stream.
*/

void
doheader(int l)
{
	KCHAR buf[1000];
	KCHAR c;
	Hptr *temp;
	char *p;
	u_int t, leftmarg, linelen;
	int m;
	
	if(headers_off) goto EXIT;
	
	for(temp=root; temp; temp=temp->next)
	{
		if(temp->linenum==l)
		{
			t=pageodd();
			if(t && temp->oddtxt) p=temp->oddtxt;
			else p=temp->eventxt;
			if(p)
			{
				m=addmacro(p);
				
				leftmarg=getleftmg('h', t);
				linelen=getrightmg('h', t)-leftmarg;
				
				c=tabfill;
				tabfill=temp->htabfill;
				pushkbuff();
				init_header();		  /* reset header attributes */
				t=fillbuff(buf, buf+arraysize(buf)-2, linelen, 0, 0, 
							&head_attrs, FMT_HEAD);
				tabfill=c;
				
				domargin(leftmarg);
				
				printline(buf, &head_attrs);
				
				while((c=getkey())!=INT_DELIM);
				pullkbuff();
				killmacro(m);
				break;
			}
		}
	}
EXIT:;
}

/* ============================================================
   Set the header font
*/

void
init_header(void)
{
	memset(&head_attrs, 0, sizeof(head_attrs));
	head_attrs.cfont=getregnum('n');
}

/* ======================================================================
	Enable/disable the printing of headers (HP=on/off)
*/

void
header_printing(void)
{
	headers_off=!getonoff();
}
