/* indents.c */

#include "vpt.h"

#define MAXDYTABS 10

static u_int	id_left,		/* left indent in MS			*/
		id_right,		/* right indent in MS			*/
		id_both,		/* double indent value in MS		*/
		id_delay,		/* delayed indent value in MS		*/
		id_temp,		/* temporary indent value in MS		*/
		id_rtemp,		/* temporary right indent in MS		*/
		id_center;		/* center indent value in MS		*/

static u_char	id_temp_ct,		/* temp ID count */
		id_temp_dl,		/* temp ID delay */
		id_rtemp_ct,		/* right temp ID count */
		id_rtemp_dl,		/* right temp ID delay */
		id_center_ct,		/* center ID count  */
		id_center_dl;		/* center ID delay  */

static char delayidflag;

static u_int dytabstack[MAXDYTABS];
static char dytabcount;

/* ============================================================
   Tokenize delayed indents
*/

void
tokidtab(void)	  /* \i0..\i9 delayed ID plus tab */
{
	KCHAR c=getkey();
	
	if(!f_isdigit(c)) unknowntab();
	returnkey(INT_DELAYID);
	returnkey((c-'0')+INT_TAB);
}

void
tokdelayid(void)   /* delayed indent command \ID */
{
	returnkey(INT_DELAYID);
}


/* ============================================================
   Set the indent values
*/

void
setbothid(void)	  /* set both indents */
{
	skipequal();
	id_both=getval(1);
}

void
restbothid(int t)  /* set both id, called from state */
      
{
	id_both=t;
}

void
setcenterid(void)  /* set center indent */
{
	skipequal();
	id_center=getval(1);
	id_center_dl=0;
	id_center_ct=1;
	if(comma_ck())
	{
		id_center_ct=getval(0);
		if(comma_ck()) id_center_dl=getval(0);
	}
}


void
setdelayid(void)  /* set delayed indent */
{
	KCHAR c;
	
	/*	delayed indents can be set with a value, or just by saying ID,
		this will just reset the flag so that ID is turned off for the
		next line.
	*/
	
	returnkey(c=getnonb());
	if(c!=CR && c!=';')
	{
		skipequal();
		id_delay=getval(1);
	}
	else
	{
		if(!delayidflag)
		{
			if(id_delay > id_left) id_left=0;	/* don't permit neg values */
			else id_left -= id_delay;
		}
	}
	delayidflag=1;
	dytabcount=0;
}


void
setleftid(void)  /* set left indent */
{
	skipequal();
	id_left=getval(1);
	dytabcount=0;
}

void
restleftid(int t)  /* set left id, called from state */
      
{
	id_left=t;
}

void
setrightid(void)   /* set right indent */
{
	skipequal();
	id_right=getval(1);
}

void
restrightid(int t)  /* set right id, called from state */
{
	id_right=t;
}

void
settempid(void)	  /* set temp indent */
{
	KCHAR c, l;
	
	l=f_tolower(c=getkey());
	if(l=='r' || l=='b')
	{
		settemprid();
		if(l=='b')
		{
			id_temp=id_rtemp;
			id_temp_dl=id_rtemp_dl;
			id_temp_ct=id_rtemp_ct;
		}
	}
	else
	{
		if(f_tolower(c)!='l') returnkey(c);
		skipequal();
		id_temp=getval(1);
		id_temp_dl=0;
		id_temp_ct=1;
		if(comma_ck())
		{
			id_temp_ct=getval(0);
			if(comma_ck()) id_temp_dl=getval(0);
		}
	}
}

void
settemprid(void)
{
	skipequal();
	id_rtemp=getval(1);
	id_rtemp_dl=0;
	id_rtemp_ct=1;
	if(comma_ck())
	{
		id_rtemp_ct=getval(0);
		if(comma_ck()) id_rtemp_dl=getval(0);
	}
}

/* ============================================================
   Get the indent values
*/


int
getbothid(void)	  /* get both indents */
{
	return id_both;
}

int
getcenterid(void)  /* ret center indent */
{
	return id_center;
}

int
getdelayid(void)  /* get delayed indent */
{
	return id_delay;
}

int
getleftid(void)  /* get left indent */
{
	return id_left;
}

int
getrightid(void)   /* get right indent */
{
	return id_right;
}

int
gettempid(void)	  /* get temp indent */
{
	if(!id_temp_dl && id_temp_ct) return id_temp;
	return 0;
}

int
gettemprid(void)
{
	if(!id_rtemp_dl && id_rtemp_ct) return id_rtemp;
	return 0;
}


/*	============================================================
	Update the delayed indent. If the flag is set we add the
	delayed indent value to the left indent.
	This is called after each line of the main text is printed.
*/

void
updatedelayid(void)
{
	if(delayidflag)
	{
		delayidflag=0;
		id_left+=id_delay;
	}
}

/*	============================================================
	The temp id command sets the delay and count. This is called
	before each line for main text is printed. The counters are
	updated and the indent value is returned. No effect on other
	margins!
*/

int
needtempid(void)
{
	if(id_temp_dl) id_temp_dl--;
	else if(id_temp_ct)
	{
		id_temp_ct--;
		return id_temp;
	}
	return 0;
}

int
needtemprid(void)
{
	if(id_rtemp_dl) id_rtemp_dl--;
	else if(id_rtemp_ct)
	{
		id_rtemp_ct--;
		return id_rtemp;
	}
	return 0;
}

int
needcenterid(void)
{
	if(id_center_dl) id_center_dl--;
	else if(id_center_ct)
	{
		id_center_ct--;
		return id_center;
	}
	return 0;
}

/*	============================================================
	Handle embedded delayed indents.
   	
   	When an indent (\ID) is found while formatting the current
   	buffer size (in microspaces) is passed and the left indent is
   	incremented. The current setting is saved on a stack.
   	
   	Note that setting a delayed indent or setting the left ID clears the
   	stack.
*/

void
add_dytab(int offset)
{
	if(dytabcount>=MAXDYTABS-1) terminate("Too many nested embedded \\IDs");
	
	dytabstack[(int)dytabcount++]=id_left;
	id_left+=offset;
}

/*	This is called by doline() after every CR is encountered at the end of
	a line. If any delayed (embedded) indents are active we check for
	a non-space following the CR. If a space follows the CR it is
	swallowed and the indent is not decremented; if anything else follows
	the CR the indent is unstacked.
*/

void
update_dytab(void)
{
	KCHAR c;
	
	if(dytabcount)
	{
		if((c=getkey())!=SPACE)
		{
			del_dytab();
			returnkey(c);
		}
	}
}

/*	This is called from update_dytab() and from fillbuff(). The fillbuff()
	call is made when a 2nd indent is found in the same line
*/

void
del_dytab(void)
{
	if(dytabcount) id_left=dytabstack[(int)--dytabcount];
}


/*	======================================================================
	Adjust indent values after a MS= command
*/

void
ms_indents(u_int n)
{
	int t;
	
	id_left=ms_adjust(id_left,   n);
	id_right=ms_adjust(id_right,  n);
	id_both=ms_adjust(id_both,   n);
	id_delay=ms_adjust(id_delay,  n);
	id_temp=ms_adjust(id_temp,   n);
	id_rtemp=ms_adjust(id_rtemp,  n);
	id_center=ms_adjust(id_center, n);
	
	for(t=0; t<dytabcount; t++)
		dytabstack[t]=ms_adjust(dytabstack[t], n);
}
