/* margins.c, set the margins and indents */

#include "vpt.h"

static u_int	leftmargin[2],  /* size of left marg in MS	 */
				rightmargin[2], /* size of right marg in MS  */
				headlmargin[2],
				headrmargin[2],
				notelmargin[2],
				notermargin[2],
				col_gutter;

static u_char	botmargin,
				topmargin;

static char *mnote_err="Illegal margin flag";

static void setmarg(u_int value, u_int *mptr, char oe);

/*	============================================================
	Set the margins from a command line..
*/

void
setlmarg(void)
{
	int value;
	char oeflag, nhflag;
	
	whichmarg(&nhflag, &oeflag);
	
	skipequal();
	value=getval(1);				  /* get value in MS */
	
	if(nhflag=='m') error(mnote_err);
	
	if(nhflag=='n')		  setmarg(value, notelmargin, oeflag);
	else if(nhflag=='h')  setmarg(value, headlmargin, oeflag);
	else 
	{
		setmarg(value, leftmargin, oeflag);
		setmarg(value, headlmargin, oeflag);
		setmarg(value, notelmargin, oeflag);
	}
}

void
setrmarg(void)
{
	int value;
	char oeflag, nhflag;
	
	whichmarg(&nhflag, &oeflag);
	
	skipequal();
	value=getval(1);
	
	if(nhflag=='m') error(mnote_err);
	
	if(nhflag=='n')		  setmarg(value, notermargin, oeflag);
	else if(nhflag=='h')  setmarg(value, headrmargin, oeflag);
	else 
	{
		setmarg(value, rightmargin, oeflag);
		setmarg(value, headrmargin, oeflag);
		setmarg(value, notermargin, oeflag);
	}
}

static void
setmarg(u_int value, u_int *mptr, char oe)
{
	if(colactive()) columnerror("Margin");
	
	switch (oe)
	{
		case 'e':
			mptr[0]=value;
			break;
			
		case 'o':
			mptr[1]=value;
			break;
			
		default:
			mptr[0]=mptr[1]=value;
	}
}

/*	============================================================
	Return the margin setting
   	
	martype='n' - footnote margin
		'h' - header margin
		
		default - main text margin
		
	 oddeven 0 - even page
			 1 - odd page
*/

int
getleftmg(char martype, char oddeven)
{
	u_int *mp;
	
	if(martype=='m') error(mnote_err);
	
	if(martype=='n')	   mp=notelmargin;
	else if(martype=='h')  mp=headlmargin;
	else				   mp=leftmargin;
	
	return mp[oddeven];
}

int
getrightmg(char martype, char oddeven)
{
	u_int *mp;
	
	if(martype=='m') error(mnote_err);
	
	if(martype=='n')	   mp=notermargin;
	else if(martype=='h')  mp=headrmargin;
	else				   mp=rightmargin;
	
	return mp[oddeven];
}
/*	============================================================
	Set the gutter size for columns
*/

void
setgutter(void)
{
	skipequal();
	col_gutter=getval(1);
}

int
getgutter(void)
{
	if (!col_gutter) col_gutter=3*msperin;
	
	return col_gutter;
}

/*	============================================================
	Determine the margin option flags. This is used by margin
	setting routine and variable expansion to determine what, if
	any, modifiers are being used.
   	
	Permitable modifiers are:
   	
		m,n,h	- margin, header, footnote
		o,e		- odd or even page
	
	Note that pointers to the variables must be passed. The options can
	be in any order.
   	
   	Eg. MLoh - left head margin for odd pages
*/

void
whichmarg(char *mtype, char *moe)
{
	KCHAR c, f;
	
	*mtype=*moe=0; /* assume no modifiers */
	
	for( ; ; )
	{
		f=f_tolower(c=getkey());
		
		if(f=='o' || f=='e')	  /* set the odd/even flag */
		{
			if(*moe) break;
			*moe=f;
		}
		
		/* set the note/header/mnote flag */
		
		else if(f=='n' || f=='h' || f=='m')
		{
			if(*mtype) break;
			*mtype=f;
		}
		
		else break;	  /* unknown option, break */
	}
	returnkey(c);
}

/*	============================================================
	Set the top/bottom margin from a command line
*/

void
settopmarg(void)
{
	if(colactive()) columnerror("Top Margin");
	
	skipequal();
	topmargin=getBval(0);
}

void
setbotmarg(void)
{
	if(colactive()) columnerror("Bottom Margin");
	
	skipequal();
	changebotmarg(getBval(1));
}

void
changebotmarg(int t)	   /* called by above & set lines page. */
               
{
	if(fnotepending() && t<botmargin)
		terminate("Illegal margin change while footnote pending");
	
	if(numcols) flushcols();
	
	botmargin=t;
}

int
gettopmarg(void)
{
	return topmargin;
}

int
getbotmarg(void)
{
	return (botmargin==0 || botmargin < topmargin || botmargin > line_page)
		? line_page : botmargin;
}

/*	============================================================
	Change the bottom margin for footnotes. 
*/

void
incbotmarg(int t)
{
	botmargin+=t;
}

void
decbotmarg(int t)
{
	botmargin-=t;
}

/*	============================================================
	Do the top margin
*/

void
dotopmargin(void)
{
	int t;
	int oldcols=numcols;
	
	numcols=0;
	
	set_spool_output();	/* set the new output spool-page file, if needed */
	
	if(getlnum()<=1) dowhen('t');
	
	if(topmargin)
	{
		while((t=getlnum())<topmargin)
		{
			doheader(t+1);
			do_full_line();
		}
	}
	
	/*	if doing columns we need this so that the first column line
		flushed is printed in the correct font. Since each column line
		ends in a FN the flush routine assumes that the state is FN, however
		printing headers can screw this up...
	*/
	
	if(oldcols) force_n_font();
	
	numcols=oldcols;
}

void
dobotmargin(void)
{
	int t;
	int oldcols=numcols;
	
	fix_partline();	 /* ensure we're on a full line */
	numcols=0;
	printfootnotes();	/* flush the footnote buffer */
	
	while((t=getlnum())<line_page)  /* print any footers */
	{
		doheader(t+1);
		do_full_line();
	}
	numcols=oldcols;
}

/*	======================================================================
	Reset the margin sizes after a MS= command
*/

void
ms_margins(int n)
{
	int t;
	
	for(t=0; t<2; t++)
	{
		leftmargin[t]=ms_adjust(leftmargin[t],  n);
		rightmargin[t]=ms_adjust(rightmargin[t], n);
		headlmargin[t]=ms_adjust(headlmargin[t], n);
		headrmargin[t]=ms_adjust(headrmargin[t], n);
		notelmargin[t]=ms_adjust(notelmargin[t], n);
		notermargin[t]=ms_adjust(notermargin[t], n);
	}
	col_gutter=ms_adjust(col_gutter, t);
}
