/* misc.c */

#include "vpt.h"
#include <ctype.h>

/*	============================================================
	Untokenize a token
*/

void
untokenize(char *p, KCHAR c)
{
	*p++=control;
	
	if(c==INT_PAD_ON)
	{
		strcpy(p, "PM+");
		p=strend(p);
	}
	else if(c==INT_PAD_OFF)
	{
		strcpy(p, "PM-");
		p=strend(p);
	}
	
	else if(c==INT_NOTE) *p++='N';
	
	else if(c==INT_GH) *p++=ghosthy;
	
	else if(c==INT_DELAYID)
	{
		*p++='I'; 
		*p++='D';
	}
	
	else if(c==INT_WIDETAB)
	{
		*p++='T'; 
		*p++='W';
	}
	
	else if(c==INT_COLTAB)
	{
		*p++='T'; 
		*p++='B';
	}
	
	else if(c==INT_CR)
	{
		*p++='C'; 
		*p++='R';
	}
	
	else if(c==INT_GH) *p++=ghosthy;
	
	else if(c==INT_HSPACE) *p++=hardspace;
	
	else if(is_attron(c))
	{
		*p++=ascregnum(c-INT_ATTRON); 
		*p++='+';
	}
	
	else if(is_attroff(c))
	{
		*p++=ascregnum(c-INT_ATTROFF); 
		*p++='-';
	}
	
	else if(is_font(c))
	{
		*p++='F'; 
		*p++=ascregnum(c-INT_FONT);
	}
	
	else if(is_dkey(c))
	{
		*p++='%'; 
		*p++=ascregnum(c-INT_DKEY);
	}
	
	else if(is_wait(c))
	{
		*p++='W'; 
		*p++=ascregnum(c-INT_DKEY);
	}
	
	else if(is_tab(c))
	{
		*p++='T'; 
		*p++=ascregnum(c-INT_TAB);
	}
	
	else if(is_tabset(c))
	{
		*p++='T'; 
		*p++='S'; 
		*p++=ascregnum(c-INT_TABSET);
	}
	
	else p--;	 /* unknown token... */
	
	*p=0;  /* terminate string */
}

/*	============================================================
	Convert an ascii name to a register number. This converts
	the ascii digits 0..9 and A..Z to 0..35. It is used to convert
	number register names, font names, attribute names, etc.
*/

int
getregnum(KCHAR n)
{
	if(f_isdigit(n))		return n-'0';
	if(f_isalpha(n))		return f_toupper(n)-'A'+10;
	return -1;
}

int
ascregnum(int n)
{
	return (n<=9) ? '0'+n : n-10+'A';
}

/*	============================================================
	Parse off ON or OFF and return 0 for off, 1 for on.
*/

int
getonoff(void)
{
	KCHAR c;
	
	skipequal();
	if(f_toupper(getnonb())!='O') goto bad;		/* has to start with O */
	if((c=f_toupper(getkey()))=='N') return	1;	/* ON, return 1 */
	if(c!='F') goto bad;
	if(f_toupper(getkey())=='F') return 0;		/* OFF, return 0 */
	
bad:
	error("'ON' or 'OFF' expected");
	
	return -1;				/* unreachable */
}

/*	============================================================
	Get/check for a valid delimiter
*/

KCHAR
getdelm(void)
{
	KCHAR c;
	
	c=isdelm(getnonb());
	if(c==0) error("Illegal delimiter");
	return c;
}


/*	======================================================================
	Adjust a unsigned int after a MS command. We pass the original value of
	the variable and the new micro spacing constant. It is assumed that
	msperin contains the old MS setting. The adjusted value is returned.
*/

u_int
ms_adjust(u_int value, u_int new_ms)
{
	if(value)
	{
		value *= new_ms;
		value /= msperin;
	}
	
	return value;
}

/* ===========================================================================
	Return pointer to end of string
*/

char *
strend(char *s)
{
	while(*s++);
	return --s;
}

/* ==============================================================================
   Character conversion routines. These are necessary since GCC doesn't return
   proper values for toupper() and tolower() when the value is >255.
*/

int
f_toupper(int c)
{
	return (c>='a' && c<='z') ? c-0x20 : c;
}

int
f_tolower(int c)
{
	return (c>='A' && c<='Z') ? c+0x20 : c;
}

/* ==============================================================================
   String conversion routines to convert entire strings to up/lowe-er case.
*/

void
lowstr(char *s)
{
	while(*s) *s++=tolower(*s);
}

void
upstr(char *s)
{
	while(*s) *s++=toupper(*s);
}

/* ==============================================================================
   Character classification routines. These are save for 16 bit values.
*/

int
f_isdigit(int c)
{
	if(c<0x100) return isdigit(c);
	else return 0;
}

int
f_isalnum(int c)
{
	if(c<0x100) return isalnum(c);
	else return 0;
}


int
f_ispunct(int c)
{
	if(c<0x100) return ispunct(c);
	else return 0;
}


int
f_isxdigit(int c)
{
	if(c<0x100) return isxdigit(c);
	else return 0;
}

int
f_isalpha(int c)
{
	if(c<0x100) return isalpha(c);
	else return 0;
}
