/* mnotes.c, marginal note routines */

#include "vpt.h"
#include "att_table.h"
#include "prefix.h"

#define MAXNOTEBUFF 2000

static char mn_temp,	/* set if temp marg note */
			mn_pos;		/* margin for note, l,r,i,o */

static u_int	mn_gutter,	 /* gutter size for notes */
				mn_size;	 /* margin size for notes */

static char	mnotetabfill=' '; /* special tab fill for mnotes */

static Att_table mn_att_table;  /* attribute/font table for notes */

static char *mnotebuff;	  /* pointer to buffer, dynamically allocated */

static void allocmnbuff(void);


/* ============================================================
   Set the marginal note margin
*/

void
setmargnote(void)
{
	KCHAR c;
	char *emsg="Illegal MN option '%c'";
	
	mn_temp=mn_pos=mn_gutter=mn_size=0;
	
	while(index("lriot", f_tolower(c=getkey())))
	{
		if(f_tolower(c)=='t')
		{
			if(mn_temp) error(emsg, c);
			mn_temp='t';
		}
		else
		{
			if(mn_pos) error(emsg, c);
			mn_pos=f_tolower(c);
		}
	}
	returnkey(c);
	
	if(!mn_pos) mn_pos='o';
	
	skipequal();
	mn_size=getval(1);  /* get size, skip optional = */
	
	if((c=getnonb())==',')
	{
		mn_gutter=getval(1);
	}
	else
	{
		returnkey(c);
		mn_gutter=msperin*3;
	}
	allocmnbuff();
}

/*	============================================================
	Strip a margin note from input and save for later printing
	
	Notes are stored in a linked list of lines.
*/

void
getmarnote(void)
{
	char *p;
	int t;
	static FLAG first;  /* flag,  set when 1st note defined */
	
	/* start off by getting the \mn[] text into the buffer */
	
	if(!mnotebuff) allocmnbuff();
	
	p=strend(mnotebuff);
	if(p>mnotebuff && *(p-1)==INT_DELIM) p--; /* overwrite INT_DELIM */
	
	t=(mnotebuff+MAXNOTEBUFF)-p;
	if(t<6) terminate("Marginal note buffer overflow");
	getstring(p, t-5, "marginal note");
	
	if(mn_size==0)		 /* no margins set so we discard the stuff we */
	{
		*mnotebuff=0;	 /* just parsed, as well as anything else buffered */
	}
	
	else
	{
		p=strend(p);
		*p++='\n';				/* trailing CR */
		*p++=INT_DELIM;			/* delim for input switching */
		*p=0;
		
		/* copy the current font if this is the first mn. */
		
		if(!first)
		{
			mn_att_table.cfont=getbodyfont();
			first++;
		}
	}
}

/*	============================================================
	Print a note. Returns the size of margin+note printed.
*/

int
printmnote(int side)
{
	KCHAR c;
	char *p, *bp, tokbuff[10], tabfillhold;
	int m_done, t_done, t, size;
	KCHAR notebuff[500], prebuff[200];
	
	tabfillhold=tabfill;
	tabfill=mnotetabfill;
	m_done=t_done=0;
	size=mn_size;
	
	if(*mnotebuff==0)
	{
		*mnotebuff='\n';
		*(mnotebuff+1)=INT_DELIM;
		*(mnotebuff+2)='\0';
	}
	pushkbuff();
	returnstr(mnotebuff);

	if(side==2) m_done=output_space(mn_gutter);
	if(mnlineprefix) size-=mnlineprefixsize;
	if(mnlinesuffix) size-=mnlinesuffixsize;
	
	t_done+=fillbuff(notebuff, notebuff+sizeof(notebuff)-2,
						size, 0, 0, &mn_att_table, FMT_MARG);
	
	if((c=getkey()) != CR) returnkey(c);
	
	/*	To delete the stuff just printed from the buffer we simply
		read text from the buffer until the terminating delimter
		and copy it over the existing buffer from the start.
	*/
	
	incomment++;
	for(p=mnotebuff, t=MAXNOTEBUFF; ; )
	{
		c=getkey();
		if(c==INT_DELIM)
		{
			*p++=c;
			*p='\0';
			break;
		}
		else if(istoken(c))
		{
			untokenize(tokbuff, c);
			bp=tokbuff;
			while(*bp) *p++=*bp++;
		}
		else *p++=c;
		if(--t==0) terminate("Marginal note buffer underflow, call Bob!");
	}
	pullkbuff();
	incomment--;
	if(*(mnotebuff)==INT_DELIM) *mnotebuff='\0';
	
	/* do line prefix... */
	
	if(mnlineprefix)
	{
		memcpy(&temp_atts, &mnsuffix_atts, sizeof(Att_table));
		returnstr(mnlineprefix);
		t=fillbuff(prebuff, prebuff+sizeof(prebuff)-2, mnlineprefixsize,
					0, 0, &temp_atts, FMT_PREFIX);
		printline(prebuff, &temp_atts);
		while(getkey() != INT_DELIM);
		t_done+=t;
		if(t<mnlineprefixsize) t_done+=output_space(mnlineprefixsize-t);
	}
	
	printline(notebuff, &mn_att_table);  /* print the  note */
	
	if(mnlinesuffix)
	{
		if(t_done<size+mnlinesuffixsize)
			t_done+=output_space((size+mnlinesuffixsize)-t_done);
		
		memcpy(&temp_atts, &mnsuffix_atts, sizeof(Att_table));
		returnstr(mnlinesuffix);
		t=fillbuff(prebuff, prebuff+sizeof(prebuff)-2, mnlinesuffixsize,
					0, 0, &temp_atts, FMT_SUFFIX);
		printline(prebuff, &temp_atts);
		while(getkey() != INT_DELIM);
		t_done+=t;
		if(t<mnlinesuffixsize) t_done+=output_space(mnlinesuffixsize-t);
	}
	
	/* flush out note area--needed only if left margin */
	
	if(side==1) m_done+=output_space(mn_gutter+(mn_size-t_done));
	
	tabfill=tabfillhold;
	return m_done+t_done;
}

/*	------------------------------------------------------------
	Alloc memory for the marginal note buffer
*/

static void
allocmnbuff(void)
{
	if((mnotebuff=malloc(MAXNOTEBUFF))==NULL)
		terminate("No memory for maginal note buffer");
	*mnotebuff='\0';  /* init buffer */
}

/*	============================================================
	Return the margin note status... This returns the marginal
	note margin size + gutter size if notes are active.
   	
	Called by doline() to determine the amount of room allocated
	for a marginal note. 
*/

int
mnotesize(void)
{
	/* If there is no margin defined return not-active */
	
	if(!mn_size) return 0;
	
	/* If tempnote flag set and nothing in buffer, return not-active */
	
	if(mn_temp && (!mnotebuff || !(*mnotebuff) )) return 0;
	
	/* Return total size of note plus gutter */
	
	return mn_size+mn_gutter;
}

/*	============================================================
	Get margin needed for a marginal note 
	Returns	0  - no note active,
			1  - note active in left margin
			2  - note active in right margin
*/

int
mnotemarg(int oddeven)
{
	int status=0;
	
	if(mnotesize())
	{
		switch(mn_pos)
		{
			case 'l': 
				status=1;	
				break;	   /* left margin */
			
			case 'r': 
				status=2;	
				break;	   /* right margin */
			
			case 'i':						 /* inner margin */
				if(oddeven) status=1;
				else status=2;
				break;
			
			case 'o':						 /* outer margin */
				if(oddeven) status=2;
				else status=1;
				break;
				
			default:
				break;	/* should never happen! */
		}
	}
	return status;
}

/*	============================================================
	Get the marginal note status. Called by math to set the MN var.
	
	0=none
	1=left
	2=right
	3=inner
	4=outer
*/

int
mnnotestatus(void)
{
	int t=0;
	
	if(mnotesize())
	{
		switch(f_tolower(mn_pos))
		{
			case 'l':
				t=1;
				break;
				
			case 'r':
				t=2;
				break;
				
			case 'i':
				t=3;
				break;
				
			case 'o':
				t=4;
				break;
				
			default:
				break;	/* should never happen */
		}
	}
	return t;
}


/*	============================================================
	Set marginal note tab fill char
*/

void
setmtabfill(void)
{
	skipequal();
	mnotetabfill=getBval(' ');
}


/*	======================================================================
	Reset margin note sizes after a MS=
*/

void
ms_mnotes(u_int n)
{
	mn_gutter=ms_adjust(mn_gutter, n);
	mn_size=ms_adjust(mn_size,   n);
}
