/* output.c	   */

#include "vpt.h"
#include "att_table.h"
#include <sys/wait.h>

static FILE *out_file;
static int dumbprint;

static char scrfprefix[50];

static FLAG p_invalid;

static char *page_spool_file;

static Att_table printer_state;

static void open_printer(void);

/*	============================================================
	Set the scratch file name prefix. Used to buffer column output
	and for endnotes.
*/

void
setsfname(void)
{
	char *p;
	
	skipequal();
	getstring(scrfprefix, sizeof(scrfprefix)-2, "scratch file prefix");
	p=strend(scrfprefix);
	if(*(p-1) != '/') strcpy(p, "/");
}

char *
getscrfname(void)
{
	return scrfprefix;
}

/*	============================================================
	Do a shell command.
*/

void
doshell(void)
{
	char buff[500];
	
	skipequal();
	getstring(buff, sizeof(buff)-2, "shell command");
	
	systemerr=system(buff);
}

/*	============================================================
	Sleep for while 
	
	This will cause a sleep for the specified number of seconds.
	This can be useful when a printer needs a delay after an int
	sequence, etc.
*/

void
zzzz(void)
{
	u_int t;
	
	skipequal();
	if((t=getval(0))!=0) sleep(t);
}


/*	============================================================
	Print a formatted line. This has a number of responsibilites....
	First, it compares what the caller wants for attributes/font with
		what the printer is set to and updates the printer.
	Second, it keeps track of the font/attribute changes in both the
		global structure (printer_state) and in the caller's table. This
		way we track the printer and the caller knows what his line has
		done.
	Third, the data is output.
	Fourth, the attributes are turned off...but the font is left selected.
*/

void
printline(KCHAR *buff, Att_table *call_atts)
            
                       /* attrs needed to start line */
{
	KCHAR c;
	int t;
	
	/*	Check all the attrs/font and set the ones which do not match
		the current state of the printer.
	*/
	
	if(call_atts->cfont!=printer_state.cfont || p_invalid)
	{
		sendFcodes(printer_state.cfont, 0);
		sendFcodes(call_atts->cfont, 1);
		printer_state.cfont=call_atts->cfont;
		p_invalid=!printpg;
	}
	
	for(t=0; t<NUM_ATTRIBUTES; t++)
	{
		if(call_atts->aselect[t]!=printer_state.aselect[t])
		{
			sendAcodes(t, call_atts->aselect[t]);
			printer_state.aselect[t]=1;
			printer_state.sizemod=call_atts->sizemod;
		}
	}
	
	/* process the line. Send expanded code sequences if encountered. */
	
	while((c=*buff++)!=0)
	{
		if(!istoken(c) && printpg)	  /* text char, send to output */
		{
			if(translate) c=translatechar(c);
			output(c);
		}
		else
		{
			if(is_attron(c))		  /* enable attribute */
			{
				c-=INT_ATTRON;
				if(printer_state.aselect[c]==0)
				{
					sendAcodes(c, 1);
					call_atts->aselect[c]=
					printer_state.aselect[c] = 1;
					if((t=getasize(c))!=0)
					{
						printer_state.sizemod=t;
						call_atts->sizemod=t;
					}
				}
			}
			else if(is_attroff(c))	 /* disable attribute */
			{
				c-=INT_ATTROFF;
				if(printer_state.aselect[c])
				{
					sendAcodes(c, 0);
					call_atts->aselect[c]=0;
					printer_state.aselect[c]=0;
					if((t=getasize(c))!=0)
					{
						printer_state.sizemod=1;
						call_atts->sizemod = 1;
					}
				}
			}
			else if(is_font(c))
			{
				c-=INT_FONT;
				if(c!=printer_state.cfont)
				{
					sendFcodes(printer_state.cfont, 0);  /* curr font off */
					sendFcodes(c, 1);				 /* enable new font */
					printer_state.cfont=c;
					call_atts->cfont=c;
				}
			}
			else if(is_dkey(c)) sendDkey(c-INT_DKEY);  /* defined key seq. */
			
			else if(is_wait(c)) dowait(c-INT_WAIT);	   /* do a wait.... */
			
			else if(c==INT_SSPACE) sendsmallspace();   /* send small space */
		}
	}
	
	/*	Turn off the attributes, but leave the font. We really have to turn
		off the attributes since blank spaces can get screwed up otherwise,
		but we'll assume that fonts can be handled. 
	*/
	
	if((printer_state.cfont != call_atts->cfont) && !printpg) p_invalid=1;
	
	for(t=0; t<NUM_ATTRIBUTES; t++)
	{
		if(printer_state.aselect[t])
		{
			sendAcodes(t, 0);
			printer_state.aselect[t]=0;
		}
	}
	printer_state.sizemod=1;
}

/*	============================================================
	Force printer state to a pica font. This is called by domargin().
	If is also called by doline() when doing multiple columns.
*/

void
force_n_font(void)
{
	int f;
	
	if( (getcsize(' ', &printer_state)!=msperin))
	{
		sendFcodes(printer_state.cfont, 0);
		f=getregnum('n');
		sendFcodes(f, 1);
		printer_state.cfont=f;
		p_invalid=!printpg;
	}
}

/*	============================================================
	Print a margin--returns the number of MS printed.
*/

int
domargin(int msize)
{
	force_n_font();  /* set to pica font */
	return output_space(msize);
}

/*	============================================================
	Output spaces. Used for margins...
*/

int
output_space(int msize)
            /* number of MS to output */
{
	int mdone=0;
	int s_size;
	
	/* do large spaces... */
	
	s_size=getcsize(' ', &printer_state);
	while(msize-s_size>=0)
	{
		if(printpg) output(' ');
		msize-=s_size;
		mdone+=s_size;
	}
	
	/* finish off with small ones */
	
	if((s_size=smallspacesize())!=0)
	{
		while(msize-s_size>=0)
		{
			sendsmallspace();
			msize-=s_size;
			mdone+=s_size;
		}
	}
	return mdone;
}

/*	============================================================
	Do a blank line
*/

void
blankline(void)
{
	int len;
	
	doeolwhens();
	
	if(numcols)				 /* this will force a padded line	  */
	{
		len=getrightmg(0, pageodd())-getleftmg(0, pageodd());
		len-=getgutter()*(numcols-1);
		len/=numcols;
		domargin(len);
		setcolline();
	}
	else do_full_line();
}

/*	============================================================
	Send a single character to the output path
*/

void
output(char c)
{
	if(printpg)
	{
		if(!out_file) open_printer(); /* if no output is open use printer  */
		
		if(numcols) colbuffer(c);
		else putc(c, out_file);
	}
}


/*	============================================================
	Change to new directory
*/

void
changedir(void)
{
	char buff[255];
	
	skipequal();
	getstring(buff, sizeof(buff)-2, "directory name");
	if((chdir(buff))==-1)
		terminate("Can't chd to '%s', %s", buff, strerror(errno));
}

/*	============================================================
	Change the output path
	
	If the filename is '@' stdout will be returned.
	If the filename is prefaced with a '-' an existing file will
		be overwritten.
	If the filename is prefaced with a '+' an existing file will
		be extended.
	If no prefix, an error will occur if the file exists.
	
*/

void
switchout(void)
{
	char *s;
	char fname[255];
	
	static FLAG file_is_pipe=0;
	
	skipequal();
	getstring(fname, sizeof(fname)-2, "output pathname");
	
	if(page_spool_file) goto EXIT2; /* ignore SO if spooling */
	
	/* close existing file */
	
	if(file_is_pipe)	// this is set if the out_file is a pipeline
	{
		pclose(out_file);
		file_is_pipe=0;
		out_file=NULL;
		wait(NULL);
	}
	if(out_file && fileno(out_file)>2) fclose(out_file);
	out_file=NULL;
	
	s=fname;
	while(*s==' ') s++;
	if(!(*s)) terminate("Output pathname required");
	
	if(*s=='|')
	{
		out_file=popen(s+1, "w");
		if(out_file) file_is_pipe=1;
		goto EXIT;
	}
	
	if(*s=='@' && *(s+1)=='\0')	  /* output to stdout... */
	{
		out_file=stdout;
		goto EXIT;
	}
	
	if(*s=='-')			 /* overwrite existing file... */
	{
		out_file=fopen(++s, "w");
		goto EXIT;
	}
	
	if(*s=='+')			/* append to existing file... */
	{
		out_file=fopen(++s, "a");
		goto EXIT;
	}
	
	/*	The default is to write to new file or existing device.
	
	*/
	
	out_file=fopen(s, "a");
	
EXIT:
	if(!out_file)
		terminate("Unable to open '%s' for output, %s", s, strerror(errno));
	
EXIT2:;
}


/* ======================================================================
	Open the default printer
*/

static void
open_printer(void)
{
	returnstr("so=|lpr\n");
	docmdline();
}

/*	============================================================
	Flush output path (if open). This is called by PI after it is
	done to ensure immediate effects
*/

void
flushoutput(void)
{
	if(out_file) fflush(out_file);
}


/*	======================================================================
	Handle dumbprinters...this is called before each line is printed.
	It forces the printer to font 'n' at the start of each line in case
	the printer cannot handle mid-line font changes.
*/

void
do_dumb_printer(Att_table *atts)
{
	if(dumbprint==1 && atts->cfont!=printer_state.cfont)
	{
		sendFcodes(printer_state.cfont, 0);
		sendFcodes(atts->cfont, 1);
		printer_state.cfont=atts->cfont;
	}
}

void
set_dumbprint(void)
{
	skipequal();
	dumbprint=getBval(0);
}

/*	============================================================
	Set the page spool file. The page spool file is changed for each
	page. The page number is appended to the filename.
*/

void
set_spool_file(char *p)
{
	if(page_spool_file)
		terminate("Only one page spool filename permitted (duplicate -s)");
	
	page_spool_file=p;
	if(!p || *p=='\n' || *p=='\0')
		terminate("Illegal spool file name");
	
	set_spool_output();
}


void
set_spool_output(void)
{
	char buff[100];
	int curr_page=getnumval('P');
	static int opened=0;
	static u_int last_page=0;
	
	if(!page_spool_file) goto EXIT;
	
	/*	the next test is needed since this the file is opened when the -s
		is parsed and again when the first page is started. Without the
		test the file for page 1 gets opened twice and any PI stuff
		before the top margin is overwritten
	*/
	
	if(out_file && opened && curr_page==last_page) goto EXIT;
	
	if(out_file)
	{
		fclose(out_file);
		out_file=NULL;
	}
	sprintf(buff, "%s.%d", page_spool_file, getnumval('P') );
	
	if((out_file=fopen(buff, "w"))==NULL)
		terminate("Can't open spool file '%s', %s", buff, strerror(errno));
	
	last_page=curr_page;
	opened=1;
	
EXIT:;
}



/*	======================================================================
	Send a softfont to the printer.
*/

void
load_font(char *f, int fn)
        	/* pathname of the font file */
       		/* font number to use */
{
	int b;
	FILE *inpath;
	
	if((inpath=fopen(f, "r"))==NULL)
		terminate("Can't open soft font '%s', %s", f, strerror(errno));
	
	if(!out_file) open_printer();	/* if no output is open use printer  */
	
	fprintf(out_file, "\x1b*c%dD", fn);	/* set font ID */
	
	while((b=fgetc(inpath))!=EOF) fputc(b, out_file);
	
	fclose(inpath);
}

