/* vpt.c */

#define MAIN
#include "vpt.h"
#include "att_table.h"
#include "error.h"
#include <getopt.h>
#include <pwd.h>
#include <signal.h>

static char *initfile;		/* set if init file found as option */

void
main(int argc, char **argv)
{
	int t;
	KCHAR c;
	char *p;
	char temperr=0;
	char buff[300];
	char firstfile=0;
	
	fprintf(stderr, "Vprint text formatter version 1.2.1 (LINUX-ELF)\n");
	fprintf(stderr,"Copyright 1992-97 Bob van der Poel Software\n");
	
	signal(SIGHUP, icepthand);
	signal(SIGINT, icepthand);
	signal(SIGQUIT, icepthand);
	
	
	/* Set the initial defaults */
	
	control	   = BSLASH;
	currline   = 0;
	dot	   = PERIOD;
	ghosthy	   = '~';
	hardspace  = '|';
	line_page  = 66;
	linespace  = 1;
	tabfill	   = ' ';
	msperin	   = MS_DEFAULT;
	
	/*	Initial initialization -- this is done by faking a command line
		string. The following are set up:
	  	
		Top/bottom margin, left/right margin, end of line string to $0d,
		change bar char to |, aux input to stdin, footnote params.
	*/
	
	returnstr("mt=6 mb=60 ml=10 mr=70 cr=$0a; cbl=[|] s? \n");
	docmdline();
	returnstr("n1=[\\s+] n2=[\\s-] n4=[. ] nf=f ns=1,1 xd=[\\cr]");
	docmdline();
	
	init_translate();
	init_header();
	init_body();
	init_nums();	  /* initialize the number registers */
	
	sprintf(buff, "%d", getpid());	/* add the current process id		*/
	add_macro_def("P_ID", buff);	/* and user id macros. These might	*/
	sprintf(buff, "%d", getuid());	/* be useful for creating temp.		*/
	add_macro_def("U_ID", buff);	/* filenames, etc.			*/
	
	/* Parse the command line */
	
	while((c=(KCHAR)getopt(argc, argv, "ef:i:o:p:s:")) != -1 )
	{
		switch(f_tolower(c))
		{
			case 'e':	   	/* set continue after error flag	 */
				temperr=1;	/* note, the REAL flag is set after  */
				break;		/* the entire command line is parsed */
			
			case 'f':		/* interpret next option as a  */
				returnkey(INT_DELIM);
				returnkey(CR);
				returnstr(optarg);	  /* command line.	 */
				docmdline();
				while(getkey()!=INT_DELIM);
				break;
			
			case 'i':			   /* specify an init file  */
				initfile=optarg;
				break;
			
			case 'o':			/* set output path */
				returnkey(INT_DELIM);
				returnkey(CR);
				returnstr(optarg);
				switchout();
				while(getkey()!=INT_DELIM);
				break;
			
			case 'p':			/* set page range(s) */
				setpagerange(optarg);
				break;
			
			case 's':			/* set a spool page file */
				set_spool_file(optarg);	/* the name of the file */
				break;
			
			default:
				usage();
				break;
		}
	}
	
	
	if(optind==argc)	// no filenames specified, use stdin
	{
		if(!newfile("@")) terminate("Can't open stdin for input");
		setinxname("vprint");
	}
	else
	{
		for ( ; optind < argc; optind++)
		{
			if(!newfile(argv[optind]))
				terminate("Can't open file '%s'", argv[optind]);
			if(!firstfile)
			{
				setinxname(argv[optind]);  /* set index filename */
				firstfile++;
			}
		}
		
	}
	
	
	/*	Read an init file. If one was specified (-i=xxx) then initfile
		will point to the param; otherwise we set initfile to the default
		init filename.
		
		A number of attempts are made to find the named file (or default). If
		the filename contains a path (ie. there is a '/' in the name)
		then only the specified path is checked. Otherwise the current directory,
		/etc/local,  and /etc are checked
	*/
	
	if(!initfile) initfile="vpt..init";
	
	for(t=0 ; ; t++)
	{
		switch(t)
		{
			case 0:
				if(index(initfile, '/')==NULL) continue;
				strcpy(buff, initfile);		/* check it only...	   */
				t=100;				/* this'll end looping */
				break;
			
			case 1:
				strcpy(buff, initfile);		/* current dir */
				break;
			
			case 2:
				p=getenv("HOME");
				if(p==NULL)
				{
					struct passwd *pw;
					pw=getpwuid(getuid());
					if(pw) p=pw->pw_dir;
				}
				if(p==NULL) continue;
				sprintf(buff, "%s/%s", p, initfile);
				break;
			
			case 3:
				sprintf(buff, "/usr/local/etc/%s", initfile);
				break;
			
			case 4:
				sprintf(buff, "/usr/etc/%s", initfile);
				break;
			
			default:
				terminate("Can't find/open init file '%s'", initfile);
		}
		if(newfile(buff)) break;  /* attempt to open, break out if found */
	}
	
	seterrflag(temperr);
	setprintflag();		/* set the print-this-page flag */
	
	/* Everything is setup, now process the files... */
	
	jumpset=1;
	setjmp(jumpbuffer);
	
	for(;;)
	{
		if((c=getkey())==0)
		{					/* eof */
			returnkey(' ');			/* dummy char for startpage() */
			startpage();			/* finish this page... */
			numcols=0;
			quit();
		}
		
		if(c==dot) docmdline();	   /* process a command line */
		else
		{
			returnkey(c);			/* process text		  */
			doline();
		}
	}
}

