/* vpt.h

   Global defines, structures and variables....
   
*/

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <malloc.h>
#include <sys/types.h>
#include <stdarg.h>
#include <unistd.h>

#ifndef MAIN
#define GLOBAL extern
#else
#define GLOBAL
#endif

#define BELL	0x07
#define CR	0x0a
#define SPACE	' '
#define PERIOD  '.'
#define BSLASH  '\\'

#define MS_DEFAULT	384

typedef short KCHAR;
typedef u_char  FLAG;

extern int errno;

/* token defs

   INT_TOKEN is used as a token delimiter. Since all large tokens
   are in the range $101..$1ff we can represent a large token in
   a character array as an INT_TOKEN, token & 0xff
*/

#define INT_TOKEN	1
#define INT_COLTAB	2
#define INT_DELIM	3
#define INT_SSPACE	4
#define INT_GH		5
#define INT_NOTE	6
#define INT_DELAYID	7
#define INT_WIDETAB	8
#define INT_CR		9
#define INT_WORDATT	0x0b
#define INT_HSPACE	0x0c

/* make sure than no more than $ff are defined here. Note that
   token $100 is not used..this makes string processing easier
   since we can use a 00 as a terminator
*/

#define NUM_ATTRIBUTES		36
#define NUM_FONTS		36
#define NUM_DKEYS		36
#define NUM_WAITS		10
#define NUM_TABS		10
#define NUM_TABSETS		10

#define INT_ATTRON		0x101
#define INT_ATTROFF		(INT_ATTRON+NUM_ATTRIBUTES)
#define INT_FONT		(INT_ATTROFF+NUM_ATTRIBUTES)
#define INT_DKEY		(INT_FONT+NUM_FONTS)
#define INT_WAIT		(INT_DKEY+NUM_FONTS)
#define INT_TAB			(INT_WAIT+NUM_WAITS)
#define INT_TABSET		(INT_TAB+NUM_TABS)
#define INT_PAD_ON		(INT_TABSET+NUM_TABS)
#define INT_PAD_OFF		(INT_PAD_ON+1)

/* These are passed around to fillbuff and justify to indicate
   the type of line being done.
*/

#define FMT_TEXT	1
#define FMT_NOTE	2
#define FMT_HEAD	3
#define FMT_MARG	4
#define FMT_CBAR	5
#define FMT_PREFIX	6
#define FMT_SUFFIX	7


/* flags */

GLOBAL FLAG	doingcmd,		/* 1==doing command line, 0==text	*/
		incomment,		/* 1==processing comment line		*/
		printoff,		/* printing on/off (off=1)		*/
		printpg,		/* print this page if 1			*/
		translate;		/* translation table on=1		*/


GLOBAL  u_char  control,		/* control char (\)			*/
		currline,		/* current page line			*/
		dot,			/* control line char (.)		*/
		ghosthy,		/* ghost hyphen character (~)		*/
		hardspace,		/* hardspace char (|)			*/
		line_page,		/* number of lines/page			*/
		linespace,		/* line spacing				*/
		halfline_on,		/* half linespaceing 0=off		*/
		numcols,		/* number of columms (1 or 2)		*/
		tabfill,		/* tab fill char (" ")			*/
		maclevel;		/* current macro processing level	*/

GLOBAL  u_int	systemerr,		/* value of last system error			*/
		partline_space,		/* part line spacing value */
		parts_per_line, 	/* # part lines per full line */
		msperin;

/*	============================================================
	Function prototypes
*/

void tokattr(KCHAR n, KCHAR mode);
void setattr(char atname, char onoff);
int getasize(int t);
void sendAcodes(int n, int onoff);
void doline(void);
void setbodyatt(KCHAR c);
int getbodyfont(void);
void set_widowlines(void);
void init_body(void);
void setcbarstatus(void);
int getcbarstatus(int oddeven);
void setcbar(void);
int docbar(int msize);
void docmdline(void);
void dopicodes(void);
int getlnum(void);
int getcurrcol(void);
int at_page_bot(void);
void flushcols(void);
void setcolline(void);
void colbuffer(int c);
void setncols(void);
int colactive(void);
void columnerror(char *s);
int get_lines_page(void);
void setcrcodes(void);
void sethfcodes(void);
void setplcodes(void);
void setlinespace(void);
void fix_partline(void);
void do_full_line(void);
void partline(void);
void tokdkey(void);
void setdefkeys(void);
void sendDkey(int n);
void ms_defkeys(int n);
void setqscodes(void);
void terminate(char *p, ...);
void error(char *p, ...);
void userr(void);
void icepthand(int signal);
void quit(void);
void seterrflag(int f);
void usage(void);
void extractnote(void);
int fnotepending(void);
void printfootnotes(void);
void setnum1(void);
void setnum2(void);
void setnum3(void);
void setnum4(void);
void setnum5(void);
void setnotefm(void);
void printnotes(void);
void setnotesp(void);
void setnoteid(int offset);
void ms_fnote(u_int n);
void tokfont(void);
void setfont(void);
void sendFcodes(int n, int onoff);
void ms_font(u_int n);
void setpadcharsize(void);
void setcharpadding(KCHAR c);
void settabcolumn(void);
void setispacing(void);
void setlineformat(void);
void getfmtstate(int *s, char **p);
int getlineformat(int mode, int pg);
void set_jcline(void);
void set_linead(void);
void set_justmode(void);
int get_justmode(void);
void ms_tabcols(u_int n);
void sethead(void);
void doheader(int l);
void init_header(void);
void header_printing(void);
void ifthen(void);
void tokidtab(void);
void tokdelayid(void);
void setbothid(void);
void restbothid(int t);
void setcenterid(void);
void setdelayid(void);
void setleftid(void);
void restleftid(int t);
void setrightid(void);
void restrightid(int t);
void settempid(void);
void settemprid(void);
int getbothid(void);
int getcenterid(void);
int getdelayid(void);
int getleftid(void);
int getrightid(void);
int gettempid(void);
int gettemprid(void);
void updatedelayid(void);
int needtempid(void);
int needtemprid(void);
int needcenterid(void);
void add_dytab(int offset);
void update_dytab(void);
void del_dytab(void);
void ms_indents(u_int n);
void setinxname(char *p);
void setindexdlm(void);
void openauxf(void);
void opentocf(void);
void openinxf(void);
void expinx(void);
void exptoc(void);
void expaux(void);
void switchin(void);
FILE *newfile(char *s);
int comma_ck(void);
void skipequal(void);
KCHAR getnonb(void);
KCHAR getkey(void);
void errlineout(void);
void openauxinp(void);
void repeatstart(void);
void repeatend(void);
int getrepeatcount(void);
void dogoto(void);
int isrbf(FILE *fpath);
int f_isprintable(KCHAR c);
int istoken(KCHAR c);
int is_attron(KCHAR c);
int is_attroff(KCHAR c);
int is_font(KCHAR c);
int is_dkey(KCHAR c);
int is_wait(KCHAR c);
int is_tab(KCHAR c);
int is_tabset(KCHAR c);
int isdelm(KCHAR c);
void set_soft_font_dir(void);
void setfmacs(FILE *f, char *n);
void defmacro(void);
void add_macro_def(char *name, u_char *text);
void expmac(void);
int addmacro(u_char *p);
KCHAR getcmac(void);
void killmacro(int n);
void maclineout(void);
int gotomacro(char *label);
void setlmarg(void);
void setrmarg(void);
int getleftmg(char martype, char oddeven);
int getrightmg(char martype, char oddeven);
void setgutter(void);
int getgutter(void);
void whichmarg(char *mtype, char *moe);
void settopmarg(void);
void setbotmarg(void);
void changebotmarg(int t);
int gettopmarg(void);
int getbotmarg(void);
void incbotmarg(int t);
void decbotmarg(int t);
void dotopmargin(void);
void dobotmargin(void);
void ms_margins(int n);
u_int getBval(int min);
u_int getAval(void);
u_int getval(char msflag);
void untokenize(char *p, KCHAR c);
int getregnum(KCHAR n);
int ascregnum(int n);
int getonoff(void);
KCHAR getdelm(void);
u_int ms_adjust(u_int value, u_int new_ms);
char *strend(char *s);
void pgifneed(u_int t);
void printinc(void);
void do_n_lines(void);
void settabfill(void);
void setmargnote(void);
void getmarnote(void);
int printmnote(int side);
int mnotesize(void);
int mnotemarg(int oddeven);
int mnnotestatus(void);
void setmtabfill(void);
void ms_mnotes(u_int n);
void init_nums(void);
void setnumval(int n, int v);
void setnumf(void);
void setnum(void);
u_int getnumval(KCHAR n);
void expnum(void);
void stringnum(KCHAR n, char *buff, int maxlen);
void setnumstr(void);
void getnumstate(int *s, char **p);
void setsfname(void);
char *getscrfname(void);
void doshell(void);
void zzzz(void);
void force_n_font(void);
int domargin(int msize);
int output_space(int msize);
void blankline(void);
void output(char c);
void changedir(void);
void switchout(void);
void flushoutput(void);
void set_dumbprint(void);
void set_spool_file(char *p);
void set_spool_output(void);
void load_font(char *f, int fn);
void setpagerange(char *s);
int pageodd(void);
void startcol(void);
void startpage(void);
void do_new_pg(void);
void setprintflag(void);
void setpbcodes(void);
void setlp(void);
void setls(void);
void ms_prefix(u_int n);
void pushkbuff(void);
void pullkbuff(void);
KCHAR getretkey(void);
void returnstr(char *s);
void returnkey(KCHAR c);
int rnd(int x);
void setss(void);
int smallspacesize(void);
void sendsmallspace(void);
void ms_smallspace(u_int n);
void statesave(void);
void staterest(void);
void ms_state(u_int n);
void getstring(char *p, int maxsize, char *emsg);
void getdstring(char *p, int maxsize, char *emsg);
void echotext(void);
int embeddedstr(KCHAR c1, KCHAR c2);
void toktab(void);
void toksettab(void);
void settab(void);
int gettabvalue(int t);
void setdytab(int t, int value);
void unknowntab(void);
void gettabstate(int *s, char **p);
void ms_tabs(u_int n);
void init_translate(void);
int translatechar(KCHAR c);
void def_trtbl(void);
void pausepage(void);
void setpauseflag(void);
void setwcodes(void);
void setwaitmsg(int t);
void tokwait(void);
void dowait(int t);
void def_when(void);
void doeolwhens(void);
void dowhen(char n);
int f_tolower(int);
int f_isdigit(int);
int f_toupper(int);
int f_isalnum(int);
int f_isxdigit(int);
int f_ispunct(int);
int f_isalpha(int);
void upstr(char *);
void lowstr(char *);
void delnotefile(void);
u_int get_current_linenumber(void);
char *get_current_filename(void);

// some common macros

#define arraysize(a) (sizeof((a))/sizeof((a[0])))
#define dswitch(a, b) switch((tolower((a))<<8)+tolower((b)))
#define dcase(a, b) case((a<<8)+b)

#undef GLOBAL
